@@ -383,21 +383,15 @@ class ImportUpstream(LogDedentMixin, GitMixin):
             self.log.info(
                 """\
                 Replacing tree contents with those from the import branch:
-                    git read-tree %s
+                    git read-tree -u --reset %s
                 """, self.import_branch)
-            self.git.read_tree(self.import_branch)
+            self.git.read_tree(self.import_branch, u=True, reset=True)
             self.log.info(
                 """\
                 Committing merge commit:
                     git commit --no-edit
                 """)
             self.git.commit(no_edit=True)
-            self.log.info(
-                """\
-                Checking out updated index:
-                    git checkout -- .
-                """)
-            self.git.checkout("--", ".")
             # finally test that everything worked correctly by comparing if
             # the tree object id's match
             if self.git.rev_parse("HEAD^{tree}") != \
@@ -504,24 +498,29 @@ class LocateChangesWalk(LocateChangesStrategy):
 
     _strategy = "drop"
 
-    def __init__(self, branch="HEAD", search_ref=None, *args, **kwargs):
+    def __init__(self, branch="HEAD", upstream="upstream/master",
+                 search_refs=None, *args, **kwargs):
+
+        if not search_refs:
+            search_refs = []
+        search_refs.insert(0, upstream)
+
         self.searcher = UpstreamMergeBaseSearcher(branch=branch,
-                                                  pattern=search_ref)
-        self.search_ref = search_ref
+                                                  patterns=search_refs)
+        self.upstream = upstream
         super(LocateChangesWalk, self).__init__(*args, **kwargs)
 
     def filtered_iter(self):
         # may wish to make class used to remove duplicate objects configurable
         # through git-upstream specific 'git config' settings
-        if self.search_ref:
-            self.filters.append(
-                DiscardDuplicateGerritChangeId(self.search_ref,
-                                               limit=self.searcher.commit))
+        self.filters.append(
+            DiscardDuplicateGerritChangeId(self.upstream,
+                                           limit=self.searcher.commit))
         self.filters.append(NoMergeCommitFilter())
         self.filters.append(ReverseCommitFilter())
         self.filters.append(DroppedCommitFilter())
         self.filters.append(
-            SupersededCommitFilter(self.search_ref,
+            SupersededCommitFilter(self.upstream,
                                    limit=self.searcher.commit))
 
         return super(LocateChangesWalk, self).filtered_iter()
@@ -548,6 +547,10 @@ class LocateChangesWalk(LocateChangesStrategy):
                 default=LocateChangesWalk.get_strategy_name(),
                 help='Use the given strategy to re-apply locally carried '
                      'changes to the import branch. (default: %(default)s)')
+@subcommand.arg('--search-refs', action='append_replace', metavar='<pattern>',
+                default=['upstream/*'], dest='search_refs',
+                help='Refs to search for previous import commit. May be '
+                     'specified multiple times.')
 @subcommand.arg('--into', dest='branch', metavar='<branch>', default='HEAD',
                 help='Branch to take changes from, and replace with imported '
                      'branch.')
@@ -585,7 +588,8 @@ def do_import(args):
 
     logger.notice("Searching for previous import")
     strategy = ImportStrategiesFactory.create_strategy(
-        args.strategy, branch=args.branch, search_ref=args.upstream_branch)
+        args.strategy, branch=args.branch, upstream=args.upstream_branch,
+        search_refs=args.search_refs)
 
     if len(strategy) == 0:
         raise ImportUpstreamError("Cannot find previous import")
@@ -595,12 +599,12 @@ def do_import(args):
     # merged.
     prev_import_merge = strategy[-1]
     if len(prev_import_merge.parents) > 1:
-        idx = next((idx for idx, commit in enumerate(prev_import_merge.parents)
-                    if commit.hexsha == strategy.searcher.commit.hexsha), None)
+        idxs = [idx for idx, commit in enumerate(prev_import_merge.parents)
+                if commit.hexsha != strategy.searcher.commit.hexsha]
 
-        if idx:
-            additional_commits = prev_import_merge.parents[idx + 1:]
-            if additional_commits and not args.branches:
+        if idxs:
+            additional_commits = [prev_import_merge.parents[i] for i in idxs]
+            if additional_commits and len(args.branches) == 0:
                 logger.warning("""\
                     **************** WARNING ****************
                     Previous import merged additional branches but non have
@@ -645,6 +649,9 @@ def do_import(args):
         if args.branches:
             for branch in args.branches:
                 logger.notice("    extra branch: '%s'", branch, dedent=False)
+        return True
+    else:
+        return False
 
 
 # vim:sw=4:sts=4:ts=4:et:
