@@ -15,17 +15,13 @@
 # limitations under the License.
 #
 
-from git_upstream.lib.utils import GitMixin
-from git_upstream.log import LogDedentMixin
-
-try:
-    from git.objects.commit import Commit
-except ImportError:
-    from git_upstream.lib.pygitcompat import GitUpstreamCompatCommit as Commit
-
 from abc import ABCMeta, abstractmethod
 import re
 
+from git_upstream.lib.utils import GitMixin
+from git_upstream.lib.pygitcompat import Commit
+from git_upstream.log import LogDedentMixin
+
 
 class Searcher(GitMixin):
     """
@@ -80,8 +76,8 @@ class Searcher(GitMixin):
         # commits to the caller
         self.log.info(
             """\
-            Walking the ancestry path between found commit and target
-                git rev-list --parents --ancestry-path %s..%s
+            Walking the changes between found commit and target
+                git rev-list --parents --simplify-merges %s..%s
             """, self.commit.hexsha, self.branch)
 
         # depending on how many commits are returned, may need to examine use
@@ -90,7 +86,7 @@ class Searcher(GitMixin):
         commit_list = Commit.iter_items(self.repo,
                                         "{0}..{1}".format(self.commit.hexsha,
                                                           self.branch),
-                                        topo_order=True, ancestry_path=True)
+                                        topo_order=True, simplify_merges=True)
 
         # chain the filters as generators so that we don't need to allocate new
         # lists for each step in the filter chain.
@@ -128,34 +124,40 @@ class UpstreamMergeBaseSearcher(LogDedentMixin, Searcher):
     base available.
     """
 
-    def __init__(self, pattern="upstream/*", search_tags=False, remotes=None,
+    def __init__(self, branch, patterns=None, search_tags=False, remotes=None,
                  *args, **kwargs):
 
+        super(UpstreamMergeBaseSearcher, self).__init__(branch, *args,
+                                                        **kwargs)
+
+        if not patterns:
+            patterns = ["upstream/*"]
+        self._patterns = patterns
+        self._references = ["refs/heads/{0}".format(ref)
+                            for ref in self.patterns]
+
         if not remotes:
             remotes = []
-        self._pattern = pattern
-        self._references = ["refs/heads/{0}".format(self.pattern)]
-
-        super(UpstreamMergeBaseSearcher, self).__init__(*args, **kwargs)
 
         if remotes:
             self._references.extend(
-                ["refs/remotes/{0}/{1}".format(s, self.pattern)
-                 for s in remotes])
+                ["refs/remotes/{0}/{1}".format(s, ref)
+                 for s in remotes for ref in self.patterns])
         else:
-            self._references.append(
-                "refs/remotes/*/{0}".format(self.pattern))
+            self._references.extend(
+                ["refs/remotes/*/{0}".format(ref) for ref in self.patterns])
 
         if search_tags:
-            self._references.append("refs/tags/{0}".format(self.pattern))
+            self._references.append(
+                ["refs/tags/{0}".format(ref) for ref in self.patterns])
 
     @property
-    def pattern(self):
+    def patterns(self):
         """
         Pattern to limit which references are searched when looking for a
         merge base commit.
         """
-        return self._pattern
+        return self._patterns
 
     def find(self):
         """
@@ -266,7 +268,7 @@ class UpstreamMergeBaseSearcher(LogDedentMixin, Searcher):
                         %s
                 """, (" \\\n" + " " * 8).join(merge_bases))
             sha1 = self.git.rev_list(
-                *merge_bases, topo_order=True, max_count=1, no_walk=True)
+                *merge_bases, topo_order=True, max_count=1)
             # now that we have the sha1, make sure to save the commit object
             self.commit = self.repo.commit(sha1)
             self.log.debug("Most recent merge-base commit is: '%s'",
@@ -280,7 +282,7 @@ class UpstreamMergeBaseSearcher(LogDedentMixin, Searcher):
     def list(self, include_all=False):
         """
         If a particular commit has been merged in mulitple times, walking the
-        ancestry path and returning all results will return all commits from
+        history and returning all results will return all commits from
         each merge point, not just the last set which are usually what is
         desired.
 
